require 'rbbr_rb.rb'
require 'kde'
include Kde

class ClassTree < ClassWalker
  def initialize(tree = nil)
    super(true)
    @p2c = Hash.new()
    @method = Hash.new()
    @constant = Hash.new()
    @items = Array.new()
  end

  def visitConstant(c)
    @constant[currentClass] << c
  end

  LAST_PRIVATE   = 1
  LAST_PROTECTED = 2
  LAST_PUBLIC    = 3
  LAST_SINGLETON = 4
  def visitPrivateMethod(m)
    ary = @method[currentClass]
    if @last != LAST_PRIVATE or currentClass != @lastClass
      ary << "--private--"
    end
    @last = LAST_PRIVATE
    @lastClass = currentClass
    ary << m
  end
  def visitProtectedMethod(m)
    ary = @method[currentClass]
    if @last != LAST_PROTECTED or currentClass != @lastClass
      ary << "--protected--"
    end
    @last = LAST_PROTECTED
    @lastClass = currentClass
    ary << m
  end
  def visitPublicMethod(m)
    ary = @method[currentClass]
    if @last != LAST_PUBLIC  or currentClass != @lastClass
      ary << "--public--"
    end
    @last = LAST_PUBLIC
    @lastClass = currentClass
    ary << m
  end
  def visitSingletonMethod(m)
    ary = @method[currentClass]
    if @last != LAST_SINGLETON or currentClass != @lastClass
      ary << "--singleton--"
    end
    @last = LAST_SINGLETON
    @lastClass = currentClass
    ary << m
  end

  def visitModule(m)
    if not m.is_a? Module # raise
      super
      return
    end

    @constant[m] = [] if @constant[m].nil?
    @method[m] = [] if @method[m].nil?

    parents = m.included_modules
    if m.is_a? Class
      parents << m.superclass
    end
    if parents.size == 0
      @p2c[m] = []
    else
      parents.each do |p|
	ary = @p2c[p]
	if ary.nil?
	  ary = []
	  @p2c[p] = ary
	end
	ary << m
      end
    end
    super

    self
  end

  def createDescendantsTree( listv )
    cc = []
    mm = []
    @p2c.each_key do |c|
      if c.type == Module
	next if c == Kernel
           mm << c
      elsif @p2c[c.superclass].nil?
	cc << c
      else
	@p2c[c.superclass] << c unless @p2c[c.superclass].index(c)
      end
    end

    cc.sort{|a,b| a.name<=>b.name}.each do |c|
      append(listv, c, true)

    end
    mm.sort{|a,b| a.name<=>b.name}.each do |c|
      append(listv, c, true)
    end
  end

  def createConstantsList(vlist,cls)
    changeConstants(vlist,cls)
  end
  def changeConstants(vlist,cls)
    vlist.clear
    if @constant[cls].is_a? Array
      @constant[cls].each do |c|
	vlist.insertItem(c)
      end
    end
    vlist
  end

  def createMethodsList(mlist,cls)
    changeMethods(mlist,cls)
  end
  def changeMethods(mlist,cls)
    mlist.clear
    if @method[cls].is_a? Array
      @method[cls].each do |c|
	mlist.insertItem(c)  #?[c]
      end
    end
    mlist
  end

  private
  def append(listv, cls, f=true)
     i = QListViewItem.new(listv,cls.name)
     @items.push(i)
    if f and @p2c[cls].is_a? Array and 0<@p2c[cls].size and cls.type != Module
      @p2c[cls].sort { |a,b| a.name <=> b.name }.each { |c|
          if c.type != Module
             if cls.type == Module and c.superclass != Object
               #do not append
             else
                append(i, c)
             end
          end
          }
    end
  end
end
